# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

module Puppet
  Type.newtype(:wls_opatch) do
    def self.module_name
      'wls_config'
    end
    include EasyType
    include Puppet_X::EnterpriseModules::WebLogic::Access
    extend Puppet_X::EnterpriseModules::WebLogic::TitleParser

    desc <<-DESC
    This resource allows you to manage opatch patches on a specific middleware home.

          wls_opatch {'/opt/oracle/middleware12c:16175470':
            ensure                  => "present",
            jdk_home_dir            => "/usr/java/jdk1.7.0_45",
            os_user                 => "oracle",
            source                  => "/vagrant/p16175470_121200_Generic.zip",
          }

    DESC

    validate do
      fail 'sub_patches may not be same as main patch id.' if sub_patches&.include?(patch_id)
    end

    ensurable

    on_create do
      source = parameters[:source].process(os_user)
      extracted_source = check_source_dir(source)
      provider.remove_incompatible_patches
      [:opatch, "apply #{extracted_source} -silent ", {}]
    end

    on_modify do
      fail 'Internal error. A patch is either there ot not. It cannot be modified.'
    end

    on_destroy do
      parameters[:source]&.process(os_user)
      check_source_dir(source)
      [:opatch, "rollback -id #{patch_id} -silent ", {}]
    end

    after_destroy do
      parameters[:source].clean
    end

    map_titles_to_attributes([
                               /^((.+):(.+))$/, [:name, :oracle_product_home_dir, :patch_id]
                             ])

    parameter :name
    parameter :patch_id
    parameter :os_user
    parameter :oracle_product_home_dir
    parameter :jdk_home_dir
    parameter :source
    parameter :orainst_dir
    parameter :tmp_dir
    parameter :remove_patches_before
    parameter :force_remove
    parameter :patch_type
    parameter :sub_patches
    parameter :logoutput

    def opatch(command, options = {})
      provider.opatch(command, options)
    end

    def check_source_dir(parent)
      patch_source_dir = Dir.glob("#{parent}/**/#{patch_id}").first
      raise "#{source} doesn't contain folder #{patch_id}" unless patch_source_dir

      patch_source_dir
    end

    def generate
      #
      # Add new resources to the catalog based on the sub_patches
      #
      return if sub_patches&.none?
      if puppet_version_lower_then('4.4.0')
        #
        # Below version 4.4.0, Puppet doesn't recognise the relations
        # between the major resource and resources generated by `generate`.
        #
        # This means all generated resources will be positioned at the end.
        # For ora_opatches, this is **not** good. We need to manage the ordering
        # of patches. Therefor we skip this function when Puppet version is to low.
        #
        raise 'Using sub_patches needs Puppet 4.4.0 or higher.'
      end

      resources = sub_patches.collect do |sub_patch|
        hash = to_hash
        hash[:name] = "#{oracle_product_home_dir}:#{sub_patch}"
        hash[:patch_id] = sub_patch
        hash[:patch_type] = 'spb-instance'
        hash[:catalog] = catalog
        hash.delete(:sub_patches)
        self.class.new(hash)
      end
      self[:ensure] = :absent
      resources
    end

    def puppet_version_lower_then(version)
      current_version = Gem::Version.new(Puppet.version)
      requested_version = Gem::Version.new(version)
      current_version <= requested_version
    end
  end
end
