# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'tempfile'
require 'fileutils'
require 'puppet_x/enterprisemodules/weblogic/settings'
require 'puppet_x/enterprisemodules/weblogic/daemon'

module Puppet_X
  module EnterpriseModules
    module WebLogic
      # Access module
      module Access
        include Settings
        include EasyType::Encryption

        DEFAULT_FILE = '/etc/wls_setting.yaml'

        def self.included(parent)
          parent.extend(Access)
        end

        def puppet_version_higher_or_equal_then(version)
          current_version = Gem::Version.new(Puppet.version)
          requested_version = Gem::Version.new(version)
          current_version >= requested_version
        end

        #
        # Based on the value of ensure, do an autorequire or an autobefore
        #
        #
        def child_of(type, variable = nil, &proc)
          autorequire(type) do
            next if instance_eval('self[:disable_autorequire]', __FILE__, __LINE__)

            begin
              present = instance_eval('self[:ensure]', __FILE__, __LINE__).to_s == 'present'
            rescue Puppet::Error
              present = true
            end
            if variable.nil? # Always
              instance_eval(&proc) if present
            elsif present
              self[variable].nil? || self[variable].to_s == 'absent' ? nil : instance_eval(&proc)
            end
          end
          if puppet_version_higher_or_equal_then('4.0.0')
            #
            # Older puppet versions don't support autobefore.
            # Because we want to be compatible with puppet 3, we will Skip
            # this functionality on older versions.
            #
            autobefore(type) do
              next if instance_eval('self[:disable_autorequire]', __FILE__, __LINE__)

              begin
                absent = instance_eval('self[:ensure]', __FILE__, __LINE__).to_s == 'absent'
              rescue Puppet::Error
                absent = false
              end
              if absent
                if variable.nil? # Always
                  instance_eval(&proc)
                elsif provider
                  #
                  # Check the value of the provider is set
                  #
                  provider.send(variable).to_s == 'absent' ? nil : full_resource(provider).instance_eval(&proc)
                end
              end
            end
          else
            Puppet.debug "Skipping setting autobefore on #{type}"
          end
        end

        def wlst(content, parameters = {})
          script = 'wlstScript'
          action = ''
          if parameters.nil?
            Puppet.debug "Executing: #{script} for a create,modify or destroy"
          else
            action = parameters['action']
            Puppet.debug "Executing: #{script} with action #{action}"
          end

          tmp_file = Tempfile.new([script, '.py'])
          tmp_file.write(content)
          tmp_file.close
          FileUtils.chmod(0o555, tmp_file.path)

          csv_string = ''
          domains = configuration

          case action
          when 'index'
            # if index do all domains
            i = 1
            domains.each do |key, values|
              Puppet.debug "domain found #{key}"
              csv_domain_string = execute_wlst(script, tmp_file, parameters, key, values, :return_output => true)
              if i > 1
                # with multi domain, remove first line if it is a header
                csv_domain_string = csv_domain_string.lines.to_a[1..-1].join
              end
              csv_string += csv_domain_string
              i += 1
            end
            convert_csv_data_to_hash(csv_string, [], :col_sep => ';')
          when 'execute'
            domains.each do |key, values|
              # check content if we do this for the right domain
              if content.include? "real_domain='#{key}"
                csv_string = execute_wlst(script, tmp_file, parameters, key, values, :return_output => true)
              else
                Puppet.debug "Skip WLST for domain #{key}"
              end
            end
            convert_csv_data_to_hash(csv_string, [], :col_sep => ';')
          else
            #  Puppet.info "domain found #{domain}"
            domains.each do |key, values|
              # check content if we do this for the right domain
              if content.include? "real_domain='#{key}"
                Puppet.debug "Got the right domain #{key} script, now execute WLST"
                execute_wlst(script, tmp_file, parameters, key, values, :return_output => false)
              else
                Puppet.debug "Skip WLST for domain #{key}"
              end
            end
          end
        end

        private

        def config_file
          Pathname.new(DEFAULT_FILE).expand_path
        end

        def execute_wlst(script, tmp_file, _parameters, domain, domain_values, options = {})
          operating_system_user = domain_values['user'] || 'oracle'
          weblogic_home_dir = domain_values['weblogic_home_dir']
          weblogic_user = domain_values['weblogic_user'] || 'weblogic'
          weblogic_connect_url = domain_values['connect_url'] || 't3://localhost:7001'
          weblogic_password          = domain_values['weblogic_password'].nil? ? '' : decrypted_value(domain_values['weblogic_password'])
          post_classpath             = domain_values['post_classpath']
          custom_trust              = domain_values['custom_trust']
          trust_keystore_file       = domain_values['trust_keystore_file']
          trust_keystore_passphrase = domain_values['trust_keystore_passphrase'].nil? ? '' : decrypted_value(domain_values['trust_keystore_passphrase'])
          debug_module              = domain_values['debug_module']
          archive_path              = domain_values['archive_path']
          return_output = options.fetch(:return_output, false)

          fail('weblogic_home_dir cannot be nil, check the wls_setting resource type') if weblogic_home_dir.nil?
          fail('weblogic_password cannot be nil, check the wls_setting resource type') if weblogic_password.nil?

          Puppet::Util::Log.level
          if ENV.fetch('WLS_CONFIG_INFO', nil) == 'debug'
            puts "Prepare to run: #{tmp_file.path},#{operating_system_user},#{domain},#{weblogic_home_dir},#{weblogic_user},#{weblogic_password},#{weblogic_connect_url}"
            puts 'vvv==================================================================='
            File.open(tmp_file.path).readlines.each do |line|
              puts line
            end
            puts '^^^===================================================================='
          end

          wls_daemon = Daemon.new(operating_system_user, domain, weblogic_home_dir, weblogic_user, weblogic_password, weblogic_connect_url, post_classpath, custom_trust, trust_keystore_file,
                                  trust_keystore_passphrase)

          if debug_module.to_s == 'true'
            FileUtils.mkdir(archive_path) unless File.directory?(archive_path)
            FileUtils.cp(tmp_file.path, archive_path)
          end

          if timeout_specified
            wls_daemon.execute_script(tmp_file.path, timeout_specified)
          else
            wls_daemon.execute_script(tmp_file.path)
          end
          File.read("/tmp/#{script}.out") if return_output
        end

        #
        # Merges the information from the provider and the type into a usable reference
        # This is used when autoprging information
        #
        def full_resource(provider)
          property_hash = provider.instance_variable_get(:@property_hash)
          self.class.parameters.each do |parameter|
            property_hash[parameter] = self[parameter]
          end
          provider
        end

        def timeout_specified
          to_hash.fetch(:timeout, nil) if respond_to?(:to_hash)
        end
      end
    end
  end
end
