# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'benchmark'
module Puppet_X
  module EnterpriseModules
    module WebLogic
      class Daemon
        include EasyType::Template
        include EasyType::Encryption

        DEFAULT_TIMEOUT = ENV.fetch('WLS_CONFIG_DAEMON_READ_TIMEOUT', nil) || 120 # 2 minutes

        def initialize(user, domain, weblogic_home_dir, weblogic_user, weblogic_password, weblogic_connect_url, post_classpath, custom_trust, trust_keystore_file, trust_keystore_passphrase)
          @user = user
          @domain = domain
          @weblogic_home_dir = weblogic_home_dir
          @weblogic_user = weblogic_user
          @weblogic_password = weblogic_password.nil? ? '' : decrypted_value(weblogic_password)
          @weblogic_connect_url = weblogic_connect_url
          @post_classpath = post_classpath
          @custom_trust = custom_trust
          @trust_keystore_file = trust_keystore_file
          @trust_keystore_passphrase = trust_keystore_passphrase.nil? ? '' : decrypted_value(trust_keystore_passphrase)

          @daemon = EasyType::Daemon.run(identity)

          return if @daemon

          if @custom_trust.to_s == 'true'
            # rubocop:disable Layout/LineLength
            @trust_parameters = "-Dweblogic.security.TrustKeyStore=CustomTrust -Dweblogic.security.CustomTrustKeyStoreFileName=#{@trust_keystore_file} -Dweblogic.security.CustomTrustKeystorePassPhrase=#{@trust_keystore_passphrase}"
            # rubocop:enable Layout/LineLength
            Puppet.debug "trust parameters #{@trust_parameters}"
          else
            Puppet.debug 'no custom trust'
          end
          start_daemon
        end

        def execute_script(script, timeout = DEFAULT_TIMEOUT)
          Puppet.debug "Executing wls-script #{script} with timeout = #{timeout}"
          time = Benchmark.realtime do
            reconnect_to_wls
            daemon.execute_command "execfile('#{script}')"
            daemon.sync(timeout)
          rescue EOFError, ThreadError
            @retries = @retries.nil? ? 1 : @retries + 1
            fail 'maximum number of retries for daemon restart exhausted.' if @retries == 3

            Puppet.debug 'Detected stopped daemon. restarting it...'
            @daemon.kill
            start_daemon
            retry
          end
          Puppet.debug "Time elapsed #{time * 1000} milliseconds on script #{script}"
        end

        private

        def identity
          "wls-#{@domain}"
        end

        def daemon
          @daemon = EasyType::Daemon.run(identity)
          return @daemon if @daemon

          start_daemon
        end

        def start_daemon
          Puppet.info "Starting the wls daemon for domain #{@domain}"
          # rubocop:disable Layout/LineLength
          command = "export POST_CLASSPATH='#{@post_classpath}';. #{@weblogic_home_dir}/server/bin/setWLSEnv.sh;java -Dweblogic.security.SSL.ignoreHostnameVerification=true #{@trust_parameters} weblogic.WLST"
          # rubocop:enable Layout/LineLength
          Puppet.debug command
          @daemon = EasyType::Daemon.new(identity, command, @user, [/reconnect\(.*\)/], %w[SyntaxError NameError])
          pass_domain
          pass_credentials
          Puppet.info "Connecting to wls on url #{@weblogic_connect_url}"
          define_common_methods
          connect_to_wls
        end

        def connect_to_wls
          daemon.execute_command "connect('#{@weblogic_user}','#{@weblogic_password}','#{@weblogic_connect_url}')"
        end

        def reconnect_to_wls
          daemon.execute_command "reconnect('#{@weblogic_user}','#{@weblogic_password}','#{@weblogic_connect_url}')"
        end

        def pass_credentials
          Puppet.debug 'Passing credentials to WLST'
          daemon.execute_command "weblogic_user = '#{@weblogic_user}'"
          daemon.execute_command "weblogic_password = '#{@weblogic_password}'"
          daemon.execute_command "weblogic_connect_url = '#{@weblogic_connect_url}'"
        end

        def pass_domain
          Puppet.debug "Passing domain #{@domain}"
          daemon.execute_command "domain = '#{@domain}'"
        end

        def define_common_methods
          Puppet.debug 'Defining common methods...'
          tmp_file = Tempfile.new('wlstCommonScript.py')
          tmp_file.write(template('wls_config/wls_config/wlst/common.py.erb', binding))
          tmp_file.close
          FileUtils.chmod(0o555, tmp_file.path)
          time = Benchmark.realtime do
            daemon.execute_command "execfile('#{tmp_file.path}')"
            daemon.sync(DEFAULT_TIMEOUT)
          end
          Puppet.debug "Time elapsed #{time * 1000} milliseconds to define common methods and connect"
        end
      end
    end
  end
end
