# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/weblogic/tree'
require 'puppet_x/enterprisemodules/weblogic/bean_path'

module Puppet_X
  module EnterpriseModules
    module WebLogic
      module Type
        # @private
        def self.included(parent)
          #
          # We use send here because in ruby 1.8.7, include is a private
          # method. In ruby 1.9 this has become public.
          # TODO: When we lose support for ruby 1.8.7 change this to more standard
          #
          parent.send(:include, EasyType)
          parent.send(:include, Puppet_X::EnterpriseModules::WebLogic::Access)
          parent.extend Puppet_X::EnterpriseModules::WebLogic::TitleParser
          parent.extend(ClassMethods)
          #
          # For all types create the parameter disable_autorequire
          #
          parent.parameter :disable_autorequire
        end

        PUPPET_META_ATTRIBUTES = [:name,
                                  :alias,
                                  :audit,
                                  :before,
                                  :loglevel,
                                  :noop,
                                  :notify,
                                  :require,
                                  :schedule,
                                  :stage,
                                  :subscribe,
                                  :tag,
                                  :provider].freeze

        def wls_config
          @wls_config ||= Puppet_X::EnterpriseModules::WebLogic::Tree.new(domain)
        end

        def pre_run_check
          fail "#{domain} is not a registered domain name." unless registered_domain?(domain)
        end

        def on_create
          Puppet.info "create #{name} "
          create_resource
          nil
        end

        def create_resource
          creation_path = BeanPath.new(bean_path.up)
          actual_creation_path = creation_path.subst(to_hash)
          fail "Parent resource #{actual_creation_path} doesn't exists." unless wls_config.exists?(actual_creation_path)

          wls_config.create(bean_type, actual_creation_path, self[key_name])
        end

        def destroy_resource
          wls_config.destroy(bean_type, bean_path.subst(to_hash), self[key_name])
        end

        def domain_name
          @domain_name ||= wls_config.get('/Name')
        end

        def to_hash
          hash = super
          hash.merge({ :domain_name => domain_name })
        end

        def on_modify
          Puppet.info "modify #{name} "
          # Let the attributes do their work
          nil
        end

        def on_destroy
          Puppet.info "destroy #{name} "
          destroy_resource
          nil
        end

        def bean_type
          self.class.bean_type
        end

        def bean_path
          self.class.bean_path
        end

        def autorequire_when(_type, type_name)
          return_value = []
          targets = self[:target] || []
          targets.each_with_index do |value, index|
            return_value << full_name(value) if self[:targettype][index] == type_name
          end
        end

        def key_name
          self.class.key_name
        end

        def registered_domain?(name)
          if catalog&.resource('wls_setting', name)
            #
            # Look in the catalog and check if there is already a wls_setting for this resource. If
            # it exists, we are running a normal(full) catalog with wls_settings done.
            #
            true
          else
            #
            # If it doesn't exist. We are either running puppet resource or a mini manifest
            # expecting WLS to run. In order to run the wls_settings has to be saved and available
            #
            self.class.configuration.key?(name)
          end
        end

        module ClassMethods
          class ExtraProps
            def initialize(parent)
              @parent = parent
            end

            def property(property_name)
              #
              # First load all extended properties like normal ones and then
              # create insync? method that will show an error when the attribute is used.
              #
              # Before we get the instances this method will be removed for enabled
              # extra properties.
              #
              @parent.property("extended_properties/#{property_name}")
              # rubocop: disable Style/DocumentDynamicEvalDefinition, Style/RedundantStringEscape
              @parent.class_eval <<-CLASS, __FILE__, __LINE__ + 1
                class #{property_name.to_s.capitalize}

                  def self.extended_attribute?
                    true
                  end

                  alias_method :original_insync?, :insync?

                  class <<self
                    alias_method :original_translate_to_resource, :translate_to_resource
                  end

                  def insync?(is)
                    fail "property '\#\{name\}' on '\#\{resource.name\}' is not enabled. Please enable it, or remove it."
                  end

                  def self.translate_to_resource(raw_resource, resource)
                    # This method returns nil
                    nil
                  end

                end

              CLASS
            end
            # rubocop: enable Style/DocumentDynamicEvalDefinition, Style/RedundantStringEscape

            # rubocop:disable Style/MissingRespondToMissing
            def method_missing(*_args)
              # rubocop:enable Style/MissingRespondToMissing
              raise ArgumentError, 'Invalid method called in extra properties block'
            end
          end

          def module_name
            'wls_config'
          end

          def execute_prefetch(resources, provider)
            name_parameter_class = paramclass(:name)
            get_raw_resources.each do |raw_resource|
              name = name_parameter_class.translate_to_resource(raw_resource) # Get the full name
              next unless resource_names_in_manifest(resources).include?(name) # Skip if not in manifest

              specified_properties = resources[name].to_hash.reject { |key, _value| PUPPET_META_ATTRIBUTES.include?(key) }.keys
              resources[name].provider = provider.map_raw_to_resource(raw_resource, specified_properties)
            end
            resources
          end

          def prefetching_for(type, resources, provider)
            domains = resources.collect { |_k, v| v.domain }
            name_parameter_class = paramclass(:name)
            domains.each do |domain|
              Puppet.info "prefetching #{type} in domain #{domain}"
              environment = { 'action' => 'execute', 'type' => type }
              raw_resources = wlst template("wls_config/#{type}/index.py.erb", binding), environment
              raw_resources.each do |raw_resource|
                name = name_parameter_class.translate_to_resource(raw_resource) # Get the full name
                next unless resource_names_in_manifest(resources).include?(name) # Skip if not in manifest

                specified_properties = resources[name].to_hash.reject { |key, _value| Puppet_X::EnterpriseModules::WebLogic::Type::PUPPET_META_ATTRIBUTES.include?(key) }.keys
                resources[name].provider = provider.map_raw_to_resource(raw_resource, specified_properties)
              end
              # rubocop:disable Lint/Void
              resources
              # rubocop:enable Lint/Void
            end
          end

          #
          # Check if the resource names are in the manifest and if the resources
          # contain any tags that are on the list of explicit list fo tags that we need
          # or explicitly on the list of tags we want to skip.
          #
          def resource_names_in_manifest(resources)
            resources.keys.select do |name|
              if Puppet['tags'].empty?
                !resources[name].tagged?(Puppet['skip_tags'])
              else
                resources[name].tagged?(Puppet['tags']) && !resources[name].tagged?(Puppet['skip_tags'])
              end
            end
          end

          def extraproperties(&block)
            @extra_properties = ExtraProps.new(self)
            @extra_properties.instance_eval(&block)
          end

          def index_condition(&condition)
            @index_condition = condition
          end

          def get_raw_resources
            Puppet.info "index #{name} "
            raw_resources
          end

          def raw_resources
            enable_extended_methods
            configuration.keys.collect do |domain|
              wls_config = Puppet_X::EnterpriseModules::WebLogic::Tree.new(domain)
              domain_name = wls_config.get('/Name')
              resources_at(domain, bean_path, { :domain_name => domain_name }, [])
            end.flatten
          end

          def key_name
            (key_attributes - [:domain, :name]).first
          end

          def resources_at(domain, path, config, resources)
            wls_config = Puppet_X::EnterpriseModules::WebLogic::Tree.new(domain)
            resulting_path = path.subst(config)
            children = wls_config.children(resulting_path)
            if path.unassigned(resulting_path).empty?
              #
              # No check if there is a callback
              #
              children = children.find_all { |c| @index_condition.call(c, resulting_path.to_s, wls_config) } if @index_condition
              resources << children.collect { |m| [{ key_name => m, :domain => domain }.merge(config)] }.flatten
            else
              children.each do |child|
                new_config = config.dup
                new_config[path.unassigned(resulting_path).first] = child
                resources_at(domain, path, new_config, resources)
              end
            end
            resources = resources.flatten
            validate_resource_names(resources)
            resources
          end

          def validate_resource_names(resources)
            resources.each do |resource|
              resource.keys.map(&:to_s).grep(/name/i).each do |key|
                unless resource[key.to_sym].scan(%r{/|:}).empty?
                  fail "WebLogic property: '#{key}' has value '#{resource[key.to_sym]}' which contains characters not support by wls_config module. Use weblogic names without a '\\' and a `:`"
                end
              end
            end
          end

          def enable_extended_methods
            enabled_extended_properties.each { |property| use_property(property) }
          end

          def enabled_extended_properties
            properties.collect(&:name).select { |p| enabled_property?(p) }
          end

          def enabled_property?(property)
            configuration.any? do |_domain, config|
              enabled_properties = config.fetch('extra_properties') { [] }
              enabled_properties.any? do |entry|
                enabled_type, enabled_property = entry.split(':')
                if !Regexp.new(enabled_type).match(name.to_s) ||
                   !Regexp.new(enabled_property).match(property.to_s) ||
                   !attrclass(property).respond_to?(:extended_attribute?) then false
                else
                  true
                end
              end
            end
          end

          def use_property(property_name)
            # rubocop: disable Style/DocumentDynamicEvalDefinition
            class_eval <<-CLASS, __FILE__, __LINE__ + 1
              class #{property_name.to_s.capitalize}

                 alias_method :insync?, :original_insync?

                 class <<self
                   alias_method :translate_to_resource, :original_translate_to_resource
                 end
              end

            CLASS
            # rubocop: enable Style/DocumentDynamicEvalDefinition
          end

          def bean_type(bean_type = nil)
            if bean_type
              @bean_type = bean_type
            else
              @bean_type
            end
          end

          def bean_path(bean_path = nil)
            if bean_path
              @bean_path = BeanPath.new(bean_path)
            elsif @bean_path
              @bean_path
            elsif @bean_type
              @bean_path ||= BeanPath.new("/#{@bean_type}s")
            else
              raise(ArgumentError, 'no bean type or path specified') unless @bean_type || @bean_path
            end
          end
        end
      end
    end
  end
end
